####################################################################;
# TRADE APPLICATION:  MULTIPLIER BOOTSTRAP; 
####################################################################;

# Empirical application of the paper "Panel Quantile Effects Via Distribution Regression"
# Authors: V. Chernozhukov, I. Fernandez-Val, M. Weidner

# Source: Helpman, Melitz, Rubinstein: "Estimating Trade Flows: Trading Partners and Trading Volumes, 2008,
# QJE, 123(2), pp. 441-487;

# Data: Helpman and Melitz web site;

# Std errors assume correct specification;

# 100% increase in distance;

# Source directory in SCC "/project/ivangrp/PDR/";

rm(list=ls());

library(parallel);
library(abind);

str.trim <- "_c99"; # "","_c99","_c95";

load(paste("EmpApp_Inf",str.trim,".RData", sep=""));
source("PDR_Source.R");

####################
# Parallel Setting #
####################

# Calculate the namber of cores
n.cores <- as.numeric(Sys.getenv("NSLOTS"));
if(is.na(n.cores))n.cores<-1;

n.boot=500;
q.trim=0.95; # 1 if str.trim=="_c95";
q.plot=c(1:which(taus==q.trim));


i.name <- all.vars(model.spec)[n.K+2];
j.name <- all.vars(model.spec)[n.K+3];



#############
# Variances #
#############

var <- colMeans(phi.all^2)/NROW(trade.data);
var.ae <- colMeans(phi.ae^2)/NROW(trade.data);

# Trade.data(i.e. phi.all) is ordered by id then by jd, now obtain the reversed ordered phi # 
phi.all.reversed <- phi.all[order(trade.data[,j.name],trade.data[,i.name]),,];
phi.ae.reversed <- phi.ae[order(trade.data[,j.name],trade.data[,i.name]),];
var.pwclus <- colMeans((phi.all+phi.all.reversed)*phi.all)/NROW(trade.data);
var.ae.pwclus <- colMeans((phi.ae+phi.ae.reversed)*phi.ae)/NROW(trade.data);
rm(phi.all.reversed,phi.ae.reversed);


########################
# Multiplier Bootstrap #
########################
boot.l <- mclapply(1:n.boot, boot.func, mc.cores=n.cores, phi=phi.all, variance=var);
boot <- do.call(abind,list(boot.l,along=3)); # (n.thres+1)*(n.K+n.I+n.J-1 + 4)*n.boot;

boot.ae.l <- mclapply(1:n.boot, boot.func, mc.cores=n.cores, phi=phi.ae, variance=var.ae);
boot.ae <- do.call(abind,list(boot.ae.l,along=3)); # 2*2*n.boot;

boot.pwclus.l <- mclapply(1:n.boot, boot.func.pwclus, mc.cores=n.cores, model.spec=model.spec, data.local=trade.data, 
                          phi=phi.all, variance=var.pwclus);
boot.pwclus <- do.call(abind,list(boot.pwclus.l,along=3));

boot.ae.pwclus.l <- mclapply(1:n.boot, boot.func.pwclus, mc.cores=n.cores, model.spec=model.spec, data.local=trade.data,
                             phi=phi.ae, variance=var.ae.pwclus);
boot.ae.pwclus <- do.call(abind,list(boot.ae.pwclus.l,along=3));
rm(phi.all,phi.ae);

t.max <- boot[n.thres+1, ,];
t.max.pwclus <- boot.pwclus[n.thres+1, ,];

t.ae <- boot.ae[2,,];
t.ae.pwclus <- boot.ae.pwclus[2,,];

t.max.qef  <- t.max[c(paste("F.",X.name[t.log2],".0",sep=""),paste("F.",X.name[t.d],".0",sep="")),];
t.max.qef1 <- t.max[c(paste("F.",X.name[t.log2],".1",sep=""),paste("F.",X.name[t.d],".1",sep="")),];
t.max.qef[t.max.qef<=t.max.qef1] <- t.max.qef1[t.max.qef<=t.max.qef1]; # 2*n.boot;
t.max.pwclus.qef  <- t.max.pwclus[c(paste("F.",X.name[t.log2],".0",sep=""),paste("F.",X.name[t.d],".0",sep="")),];
t.max.pwclus.qef1 <- t.max.pwclus[c(paste("F.",X.name[t.log2],".1",sep=""),paste("F.",X.name[t.d],".1",sep="")),];
t.max.pwclus.qef[t.max.pwclus.qef<=t.max.pwclus.qef1] <- t.max.pwclus.qef1[t.max.pwclus.qef<=t.max.pwclus.qef1];

#save.image(paste("EmpApp_CB",str.trim,".RData", sep=""));

alpha <- 0.05;

crt <- apply(t.max, 1, quantile, 1-alpha, na.rm=T); 
crt.pwclus <- apply(t.max.pwclus, 1, quantile, 1-alpha, na.rm=T); 

crt.qef <- apply(t.max.qef, 1, quantile, 1-alpha, na.rm=T); 
crt.pwclus.qef <- apply(t.max.pwclus.qef, 1, quantile, 1-alpha, na.rm=T);

crt.ae <- apply(t.ae, 1, quantile, 1-alpha, na.rm=T);
crt.pwclus.ae <- apply(t.ae.pwclus, 1, quantile, 1-alpha, na.rm=T);


# Bounds without pairwise clustering #

u.bound <- e.beta[,paste("bc.",X.name,sep="")]+t(crt*sqrt(var))[,X.name]; # point crt = qnorm(1-alpha/2);
l.bound <- e.beta[,paste("bc.",X.name,sep="")]-t(crt*sqrt(var))[,X.name];

F.name <- c(paste("F.",X.name[t.log2],".0",sep=""),paste("F.",X.name[t.log2],".1",sep=""),
            paste("F.",X.name[t.d],".0",sep=""),paste("F.",X.name[t.d],".1",sep=""))
u.bound.F <- apply(e.F[,paste(F.name,".bc",sep="")]+t(crt*sqrt(var))[,F.name], 2,sort); # point crt = qnorm(1-alpha/2);
l.bound.F <- apply(e.F[,paste(F.name,".bc",sep="")]-t(crt*sqrt(var))[,F.name], 2,sort);

u.bound.F[u.bound.F > 1] <- 1;
u.bound.F[u.bound.F < 0] <- 0;
l.bound.F[l.bound.F > 1] <- 1;
l.bound.F[l.bound.F < 0] <- 0;


u.bound.F.qef <- apply(e.F[,paste(F.name,".bc",sep="")]+
                         t(c(rep(crt.qef[1:length(t.log2)],2),
                             rep(crt.qef[(length(t.log2)+1):(length(t.log2)+length(t.d))],2))*sqrt(var)[F.name,]), 
                       2,sort); # point crt = qnorm(1-alpha/2);
l.bound.F.qef <- apply(e.F[,paste(F.name,".bc",sep="")]-
                         t(c(rep(crt.qef[1:length(t.log2)],2),
                             rep(crt.qef[(length(t.log2)+1):(length(t.log2)+length(t.d))],2))*sqrt(var)[F.name,]), 
                       2,sort);

u.bound.F.qef[u.bound.F.qef > 1] <- 1;
u.bound.F.qef[u.bound.F.qef < 0] <- 0;
l.bound.F.qef[l.bound.F.qef > 1] <- 1;
l.bound.F.qef[l.bound.F.qef < 0] <- 0;




# Bounds with pairwise clustering #

u.bound.pwclus <- e.beta[,paste("bc.",X.name,sep="")]+t(crt.pwclus*sqrt(var.pwclus))[,X.name]; # point crt = qnorm(1-alpha/2);
l.bound.pwclus <- e.beta[,paste("bc.",X.name,sep="")]-t(crt.pwclus*sqrt(var.pwclus))[,X.name];

u.bound.F.pwclus <- apply(e.F[,paste(F.name,".bc",sep="")]+t(crt.pwclus*sqrt(var.pwclus))[,F.name], 2,sort); # point crt = qnorm(1-alpha/2);
l.bound.F.pwclus <- apply(e.F[,paste(F.name,".bc",sep="")]-t(crt.pwclus*sqrt(var.pwclus))[,F.name], 2,sort);

u.bound.F.pwclus[u.bound.F.pwclus > 1] <- 1;
u.bound.F.pwclus[u.bound.F.pwclus < 0] <- 0;
l.bound.F.pwclus[l.bound.F.pwclus > 1] <- 1;
l.bound.F.pwclus[l.bound.F.pwclus < 0] <- 0;


u.bound.F.pwclus.qef <- apply(e.F[,paste(F.name,".bc",sep="")]+
                         t(c(rep(crt.pwclus.qef[1:length(t.log2)],2),
                             rep(crt.pwclus.qef[(length(t.log2)+1):(length(t.log2)+length(t.d))],2))*
                             sqrt(var.pwclus)[F.name,]), 2,sort); # point crt = qnorm(1-alpha/2);
l.bound.F.pwclus.qef <- apply(e.F[,paste(F.name,".bc",sep="")]-
                         t(c(rep(crt.pwclus.qef[1:length(t.log2)],2),
                             rep(crt.pwclus.qef[(length(t.log2)+1):(length(t.log2)+length(t.d))],2))*
                             sqrt(var.pwclus)[F.name,]), 2,sort);

u.bound.F.pwclus.qef[u.bound.F.pwclus.qef > 1] <- 1;
u.bound.F.pwclus.qef[u.bound.F.pwclus.qef < 0] <- 0;
l.bound.F.pwclus.qef[l.bound.F.pwclus.qef > 1] <- 1;
l.bound.F.pwclus.qef[l.bound.F.pwclus.qef < 0] <- 0;


#########################
# Distribution Function #
#########################
DF <- apply(e.F, 2, approxfun, x=ys, rule=2:1);
u.df <- apply(u.bound.F, 2, approxfun, x=ys, rule=2:1);
l.df <- apply(l.bound.F, 2, approxfun, x=ys, rule=2:1);
u.df.pwclus <- apply(u.bound.F.pwclus, 2, approxfun, x=ys, rule=2:1);
l.df.pwclus <- apply(l.bound.F.pwclus, 2, approxfun, x=ys, rule=2:1);

#save.image(paste("EmpApp_CB",str.trim,".RData", sep=""));



############################
# Quantile Effect Function #
############################
QF <- apply(e.F, 2, approxfun, y=ys, rule=2:1); # a list of function objects;
u.qef <- apply(u.bound.F.qef, 2, approxfun, y=ys, rule=2:1);
l.qef <- apply(l.bound.F.qef, 2, approxfun, y=ys, rule=2:1);
u.qef.pwclus <- apply(u.bound.F.pwclus.qef, 2, approxfun, y=ys, rule=2:1);
l.qef.pwclus <- apply(l.bound.F.pwclus.qef, 2, approxfun, y=ys, rule=2:1);

#save.image(paste("EmpApp_CB",str.trim,".RData", sep=""));


######################
# Poisson Regression #
######################
pois.model <- glm(model.spec, trade.data, family = poisson(link = "log")); #family = quasipoisson?
lin.pois   <- pois.model$linear.predictors;
theta.pois <- pois.model$coeff;

e.F.pois.l <- mclapply(1:n.thres, PDR.F.poisson, mc.cores=n.cores, model.spec=model.spec, data.local=trade.data, 
                       ys=ys, theta.pois=theta.pois, lin.pois=lin.pois, t.log2=t.log2, t.d=t.d);
e.F.pois <- do.call(rbind, e.F.pois.l); #n.thres*4;

QF.pois <- apply(e.F.pois, 2, approxfun, y=ys, rule=2:1);



###################
# Average Effects #
###################
para.name <- X.name[c(t.log2,t.d)];
avg.eff <- rep(0,length(para.name));
avg.eff.bc <- rep(0,length(para.name));
for(i in 1:length(para.name)){
  avg.eff[i]    <- integrate(DF[[paste("F.",para.name[i],".0",sep="")]], ys[1], ys[n.thres])$value - 
                   integrate(DF[[paste("F.",para.name[i],".1",sep="")]], ys[1], ys[n.thres])$value;
  avg.eff.bc[i] <- integrate(DF[[paste("F.",para.name[i],".0.bc",sep="")]], ys[1], ys[n.thres])$value - 
                   integrate(DF[[paste("F.",para.name[i],".1.bc",sep="")]], ys[1], ys[n.thres])$value;
};
names(avg.eff)    <- para.name;
names(avg.eff.bc) <- para.name;


# Trimmed Average Effect #
trimmed.avg.eff <- rep(0,length(para.name));
trimmed.avg.eff.bc <- rep(0,length(para.name));
for(i in 1:length(para.name)){
  trimmed.avg.eff[i]    <- ys[which(taus==q.trim)]*(1/DF[[paste("F.",para.name[i],".1",sep="")]](ys[which(taus==q.trim)])-1/DF[[paste("F.",para.name[i],".0",sep="")]](ys[which(taus==q.trim)])) +
    integrate(DF[[paste("F.",para.name[i],".0",sep="")]], ys[1], ys[which(taus==q.trim)])$value/DF[[paste("F.",para.name[i],".0",sep="")]](ys[which(taus==q.trim)]) - 
    integrate(DF[[paste("F.",para.name[i],".1",sep="")]], ys[1], ys[which(taus==q.trim)])$value/DF[[paste("F.",para.name[i],".1",sep="")]](ys[which(taus==q.trim)]);
  trimmed.avg.eff.bc[i]    <- ys[which(taus==q.trim)]*(1/DF[[paste("F.",para.name[i],".1.bc",sep="")]](ys[which(taus==q.trim)])-1/DF[[paste("F.",para.name[i],".0.bc",sep="")]](ys[which(taus==q.trim)])) +
    integrate(DF[[paste("F.",para.name[i],".0.bc",sep="")]], ys[1], ys[which(taus==q.trim)])$value/DF[[paste("F.",para.name[i],".0.bc",sep="")]](ys[which(taus==q.trim)]) - 
    integrate(DF[[paste("F.",para.name[i],".1.bc",sep="")]], ys[1], ys[which(taus==q.trim)])$value/DF[[paste("F.",para.name[i],".1.bc",sep="")]](ys[which(taus==q.trim)]);
  
};
names(trimmed.avg.eff)    <- para.name;
names(trimmed.avg.eff.bc) <- para.name;

# Average Effects by Poisson #
avg.eff.pois <- rep(0,length(para.name));
avg.eff.pois.df <- rep(0,length(para.name));
DF.pois <- apply(e.F.pois, 2, approxfun, x=ys, rule=2:1);
for(i in 1:length(t.log2)){
  avg.eff.pois[i] <- mean(exp(lin.pois + log(2)*theta.pois[t.log2[i]])[trade.data[,Y.name]<=ys[n.thres]]) - 
                     mean(exp(lin.pois)[trade.data[,Y.name]<=ys[n.thres]]);
  
  avg.eff.pois.df[i] <- integrate(DF.pois[[paste("F.",para.name[i],".0",sep="")]], ys[1], ys[n.thres])$value - 
    integrate(DF.pois[[paste("F.",para.name[i],".1",sep="")]], ys[1], ys[n.thres])$value;
};
for(i in 1:length(t.d)){
  avg.eff.pois[length(t.log2)+i] <- mean(exp(lin.pois + (1-trade.data[,X.name[t.d[i]]])*theta.pois[t.d[i]])[trade.data[,Y.name]<=ys[n.thres]]) -
                                    mean(exp(lin.pois - trade.data[,X.name[t.d[i]]]*theta.pois[t.d[i]])[trade.data[,Y.name]<=ys[n.thres]]);
  avg.eff.pois.df[length(t.log2)+i] <- integrate(DF.pois[[paste("F.",para.name[length(t.log2)+i],".0",sep="")]], ys[1], ys[n.thres])$value - 
    integrate(DF.pois[[paste("F.",para.name[length(t.log2)+i],".1",sep="")]], ys[1], ys[n.thres])$value;
};

# Trimmed Average Effect by Poisson #
trimmed.avg.eff.pois.df <- rep(0,length(para.name));
for(i in 1:length(para.name)){
  trimmed.avg.eff.pois.df[i]    <- ys[which(taus==q.trim)]*(1/DF.pois[[paste("F.",para.name[i],".1",sep="")]](ys[which(taus==q.trim)])-1/DF.pois[[paste("F.",para.name[i],".0",sep="")]](ys[which(taus==q.trim)])) +
    integrate(DF.pois[[paste("F.",para.name[i],".0",sep="")]], ys[1], ys[which(taus==q.trim)])$value/DF.pois[[paste("F.",para.name[i],".0",sep="")]](ys[which(taus==q.trim)]) - 
    integrate(DF.pois[[paste("F.",para.name[i],".1",sep="")]], ys[1], ys[which(taus==q.trim)])$value/DF.pois[[paste("F.",para.name[i],".1",sep="")]](ys[which(taus==q.trim)]);
};
names(trimmed.avg.eff.pois.df)    <- para.name;


################################
# Average Effects by Summation #
################################
para.name <- X.name[c(t.log2,t.d)];
ae <- rep(0,length(para.name));
ae.bc <- rep(0,length(para.name));
for(i in 1:length(para.name)){
  ae[i] <- sum((1-e.F[-length(ys), paste("F.",para.name[i],".1",sep="")])*(ys[-1] - ys[-length(ys)])) - 
    sum((1-e.F[-length(ys), paste("F.",para.name[i],".0",sep="")])*(ys[-1] - ys[-length(ys)]));
  ae.bc[i] <- sum((1-e.F[-length(ys), paste("F.",para.name[i],".1.bc",sep="")])*(ys[-1] - ys[-length(ys)])) - 
    sum((1-e.F[-length(ys), paste("F.",para.name[i],".0.bc",sep="")])*(ys[-1] - ys[-length(ys)]));
};
names(ae)    <- para.name;
names(ae.bc) <- para.name;


# Trimmed Average Effect #
trimmed  <- c((which(taus==q.trim)+1):n.thres);
tae <- rep(0,length(para.name));
tae.bc <- rep(0,length(para.name));
for(i in 1:length(para.name)){
  tae[i] <- sum((1-e.F[-c(trimmed,min(trimmed)-1), paste("F.",para.name[i],".1",sep="")])*
               (ys[-c(1,trimmed)] - ys[-c(trimmed,min(trimmed)-1)]))/e.F[min(trimmed)-1, paste("F.",para.name[i],".1",sep="")] - 
    sum((1-e.F[-c(trimmed,min(trimmed)-1),  paste("F.",para.name[i],".0",sep="")])*
          (ys[-c(1,trimmed)] - ys[-c(trimmed,min(trimmed)-1)]))/e.F[min(trimmed)-1, paste("F.",para.name[i],".0",sep="")];
  tae.bc[i] <- sum((1-e.F[-c(trimmed,min(trimmed)-1), paste("F.",para.name[i],".1.bc",sep="")])*
               (ys[-c(1,trimmed)] - ys[-c(trimmed,min(trimmed)-1)]))/e.F[min(trimmed)-1, paste("F.",para.name[i],".1.bc",sep="")] - 
    sum((1-e.F[-c(trimmed,min(trimmed)-1),  paste("F.",para.name[i],".0.bc",sep="")])*
          (ys[-c(1,trimmed)] - ys[-c(trimmed,min(trimmed)-1)]))/e.F[min(trimmed)-1, paste("F.",para.name[i],".0.bc",sep="")];
};
names(tae)    <- para.name;
names(tae.bc) <- para.name;

# Average Effects by Poisson #
ae.pois.df <- rep(0,length(para.name));
for(i in 1:length(para.name)){
  ae.pois.df[i]    <- sum((1-e.F.pois[-length(ys), paste("F.",para.name[i],".1",sep="")])*(ys[-1] - ys[-length(ys)])) - 
    sum((1-e.F.pois[-length(ys), paste("F.",para.name[i],".0",sep="")])*(ys[-1] - ys[-length(ys)]));
};
names(ae.pois.df) <- para.name;

# Trimmed Average Effect by Poisson #
tae.pois.df <- rep(0,length(para.name));
for(i in 1:length(para.name)){
  tae.pois.df[i] <- sum((1-e.F.pois[-c(trimmed,min(trimmed)-1), paste("F.",para.name[i],".1",sep="")])*
                  (ys[-c(1,trimmed)] - ys[-c(trimmed,min(trimmed)-1)]))/e.F.pois[min(trimmed)-1, paste("F.",para.name[i],".1",sep="")] - 
    sum((1-e.F.pois[-c(trimmed,min(trimmed)-1),  paste("F.",para.name[i],".0",sep="")])*
          (ys[-c(1,trimmed)] - ys[-c(trimmed,min(trimmed)-1)]))/e.F.pois[min(trimmed)-1, paste("F.",para.name[i],".0",sep="")];
};
names(tae.pois.df) <- para.name;


ae.table <- rbind(c(avg.eff,ae),
                  c(avg.eff.bc,ae.bc),
                  c(avg.eff.pois.df,ae.pois.df),
                  
                  c(trimmed.avg.eff, tae),
                  c(trimmed.avg.eff.bc, tae.bc),
                  c(trimmed.avg.eff.pois.df, tae.pois.df),
                  c(avg.eff.pois,NA,NA));
rownames(ae.table) <- c("Uncorrected DF","Bias Corrected DF","Poisson DF",
                        "Uncorrected DF trimmed","Bias Corrected DF trimmed","Poisson DF trimmed","Poisson Mean");



u.bound.ae <- avg.eff.bc + crt.ae*sqrt(var.ae);
l.bound.ae <- avg.eff.bc - crt.ae*sqrt(var.ae);

u.bound.ae.pwclus <- avg.eff.bc + crt.pwclus.ae*sqrt(var.ae.pwclus);
l.bound.ae.pwclus <- avg.eff.bc - crt.pwclus.ae*sqrt(var.ae.pwclus);


save.image(paste("EmpApp_CB",str.trim,".RData", sep=""));


##################
# Print AE Table #
##################
library(xtable);
options(xtable.floating=FALSE);
options(xtable.timestamp="");

xtable(ae.table, digits=2);


###############
# Plot Graphs #
###############
#par.current <- par();

pdf(paste("Trade_coeff",str.trim,".pdf",sep=""));
  para.name <- X.name[c(t.log2,t.d)];
  ylimlist <- rbind(c(1,2.1),c(-1.8,0.1));
  
  # Without pairwise clustering #
  matplot(taus[q.plot],
          cbind(u.bound[q.plot,paste("bc.",para.name[1],sep="")],
                e.beta[q.plot,c(para.name[1],paste("bc.",para.name[1],sep=""))],
                l.bound[q.plot,paste("bc.",para.name[1],sep="")]),
          col=c("black","black","red","black"),lty=c(2,1,1,2),type="l",ylim=ylimlist[1,],
          xlab="Quantile of Trade",ylab="DR Coeff",main="Log Distance");
  legend("topleft",legend=c("Uncorrected","Bias corrected","95% CB"),col=c("black","red","black"),lty=c(1,1,2));
  matplot(taus[q.plot],
          cbind(u.bound[q.plot,paste("bc.",para.name[2],sep="")],
                e.beta[q.plot,c(para.name[2],paste("bc.",para.name[2],sep=""))],
                l.bound[q.plot,paste("bc.",para.name[2],sep="")]),
          col=c("black","black","red","black"),lty=c(2,1,1,2),type="l",ylim=ylimlist[2,],
          xlab="Quantile of Trade",ylab="DR Coeff",main="Legal");
  legend("bottomleft",legend=c("Uncorrected","Bias corrected","95% CB"),col=c("black","red","black"),lty=c(1,1,2));
  
  
  # With pairwise clustering #
  matplot(taus[q.plot],
          cbind(u.bound.pwclus[q.plot,paste("bc.",para.name[1],sep="")],
                e.beta[q.plot,c(para.name[1],paste("bc.",para.name[1],sep=""))],
                l.bound.pwclus[q.plot,paste("bc.",para.name[1],sep="")]),
          col=c("black","black","red","black"),lty=c(2,1,1,2),type="l",ylim=ylimlist[1,],
          xlab="Quantile of Trade",ylab="DR Coeff",main="Log Distance with pw Clustering");
  legend("topleft",legend=c("Uncorrected","Bias corrected","95% CB"),col=c("black","red","black"),lty=c(1,1,2));
  matplot(taus[q.plot],
          cbind(u.bound.pwclus[q.plot,paste("bc.",para.name[2],sep="")],
                e.beta[q.plot,c(para.name[2],paste("bc.",para.name[2],sep=""))],
                l.bound.pwclus[q.plot,paste("bc.",para.name[2],sep="")]),
          col=c("black","black","red","black"),lty=c(2,1,1,2),type="l",ylim=ylimlist[2,],
          xlab="Quantile of Trade",ylab="DR Coeff",main="Legal with pw Clustering");
  legend("bottomleft",legend=c("Uncorrected","Bias corrected","95% CB"),col=c("black","red","black"),lty=c(1,1,2));

dev.off();

pdf(paste("Trade_dfqf",str.trim,".pdf",sep=""));
  para.name <- X.name[c(t.log2,t.d)];
  
  # Without pairwise clustering #
  
  matplot(ys[q.plot],
          cbind(u.bound.F[q.plot,paste("F.",para.name[1],".0.bc",sep="")],
                e.F[q.plot,paste("F.",para.name[1],".0.bc",sep="")],
                l.bound.F[q.plot,paste("F.",para.name[1],".0.bc",sep="")],
                u.bound.F[q.plot,paste("F.",para.name[1],".1.bc",sep="")],
                e.F[q.plot,paste("F.",para.name[1],".1.bc",sep="")],
                l.bound.F[q.plot,paste("F.",para.name[1],".1.bc",sep="")],
                1),
          col=c("black","black","black","red","red","red","grey"),lty=c(2,1,2,2,1,2,2),type="l",ylim=c(0.5,1),
          xlab="Volume of Trade",ylab="Probability",main="Log Distance");
  legend("bottomright",legend=c("Observed Distance","2*Distance","95% CB","95% CB"),
         col=c("black","red","black","red"),lty=c(1,1,2,2));
  matplot(ys[q.plot],
          cbind(u.bound.F[q.plot,paste("F.",para.name[2],".0.bc",sep="")],
                e.F[q.plot,paste("F.",para.name[2],".0.bc",sep="")],
                l.bound.F[q.plot,paste("F.",para.name[2],".0.bc",sep="")],
                u.bound.F[q.plot,paste("F.",para.name[2],".1.bc",sep="")],
                e.F[q.plot,paste("F.",para.name[2],".1.bc",sep="")],
                l.bound.F[q.plot,paste("F.",para.name[2],".1.bc",sep="")],
                1),
          col=c("black","black","black","red","red","red","grey"),lty=c(2,1,2,2,1,2,2),type="l",ylim=c(0.5,1),
          xlab="Volume of Trade",ylab="Probability",main="Legal");
  legend("bottomright",legend=c("Legal=0","Legal=1","95% CB","95% CB"),
         col=c("black","red","black","red"),lty=c(1,1,2,2));
  
  matplot(cbind(u.bound.F[q.plot,paste("F.",para.name[1],".0.bc",sep="")],
                e.F[q.plot,paste("F.",para.name[1],".0.bc",sep="")],
                l.bound.F[q.plot,paste("F.",para.name[1],".0.bc",sep="")],
                u.bound.F[q.plot,paste("F.",para.name[1],".1.bc",sep="")],
                e.F[q.plot,paste("F.",para.name[1],".1.bc",sep="")],
                l.bound.F[q.plot,paste("F.",para.name[1],".1.bc",sep="")]),
          ys[q.plot],col=c("black","black","black","red","red","red"),lty=c(2,1,2,2,1,2),type="l",
          xlab="Quantile Index",ylab="Volume of Trade",main="Log Distance");
  legend("topleft",legend=c("Observed Distance","2*Distance","95% CB","95% CB"),
         col=c("black","red","black","red"),lty=c(1,1,2,2));
  matplot(cbind(u.bound.F[q.plot,paste("F.",para.name[2],".0.bc",sep="")],
                e.F[q.plot,paste("F.",para.name[2],".0.bc",sep="")],
                l.bound.F[q.plot,paste("F.",para.name[2],".0.bc",sep="")],
                u.bound.F[q.plot,paste("F.",para.name[2],".1.bc",sep="")],
                e.F[q.plot,paste("F.",para.name[2],".1.bc",sep="")],
                l.bound.F[q.plot,paste("F.",para.name[2],".1.bc",sep="")]),
          ys[q.plot],col=c("black","black","black","red","red","red"),lty=c(2,1,2,2,1,2),type="l",
          xlab="Quantile Index",ylab="Volume of Trade",main="Legal");
  legend("topleft",legend=c("Legal=0","Legal=1","95% CB","95% CB"),
         col=c("black","red","black","red"),lty=c(1,1,2,2));
  
  
  # With pairwise clustering #
  
  matplot(ys[q.plot],
          cbind(u.bound.F.pwclus[q.plot,paste("F.",para.name[1],".0.bc",sep="")],
                e.F[q.plot,paste("F.",para.name[1],".0.bc",sep="")],
                l.bound.F.pwclus[q.plot,paste("F.",para.name[1],".0.bc",sep="")],
                u.bound.F.pwclus[q.plot,paste("F.",para.name[1],".1.bc",sep="")],
                e.F[q.plot,paste("F.",para.name[1],".1.bc",sep="")],
                l.bound.F.pwclus[q.plot,paste("F.",para.name[1],".1.bc",sep="")],
                1),
          col=c("black","black","black","red","red","red","grey"),lty=c(2,1,2,2,1,2,2),type="l",ylim=c(0.5,1),
          xlab="Volume of Trade",ylab="Probability",main="Log Distance with pw Clustering");
  legend("bottomright",legend=c("Observed Distance","2*Distance","95% CB","95% CB"),
         col=c("black","red","black","red"),lty=c(1,1,2,2));
  matplot(ys[q.plot],
          cbind(u.bound.F.pwclus[q.plot,paste("F.",para.name[2],".0.bc",sep="")],
                e.F[q.plot,paste("F.",para.name[2],".0.bc",sep="")],
                l.bound.F.pwclus[q.plot,paste("F.",para.name[2],".0.bc",sep="")],
                u.bound.F.pwclus[q.plot,paste("F.",para.name[2],".1.bc",sep="")],
                e.F[q.plot,paste("F.",para.name[2],".1.bc",sep="")],
                l.bound.F.pwclus[q.plot,paste("F.",para.name[2],".1.bc",sep="")],
                1),
          col=c("black","black","black","red","red","red","grey"),lty=c(2,1,2,2,1,2,2),type="l",ylim=c(0.5,1),
          xlab="Volume of Trade",ylab="Probability",main="Legal with pw Clustering");
  legend("bottomright",legend=c("Legal=0","Legal=1","95% CB","95% CB"),
         col=c("black","red","black","red"),lty=c(1,1,2,2));
  
  matplot(cbind(u.bound.F.pwclus[q.plot,paste("F.",para.name[1],".0.bc",sep="")],
                e.F[q.plot,paste("F.",para.name[1],".0.bc",sep="")],
                l.bound.F.pwclus[q.plot,paste("F.",para.name[1],".0.bc",sep="")],
                u.bound.F.pwclus[q.plot,paste("F.",para.name[1],".1.bc",sep="")],
                e.F[q.plot,paste("F.",para.name[1],".1.bc",sep="")],
                l.bound.F.pwclus[q.plot,paste("F.",para.name[1],".1.bc",sep="")]),
          ys[q.plot],col=c("black","black","black","red","red","red"),lty=c(2,1,2,2,1,2),type="l",
          xlab="Quantile Index",ylab="Volume of Trade",main="Log Distance with pw Clustering");
  legend("topleft",legend=c("Observed Distance","2*Distance","95% CB","95% CB"),
         col=c("black","red","black","red"),lty=c(1,1,2,2));
  matplot(cbind(u.bound.F.pwclus[q.plot,paste("F.",para.name[2],".0.bc",sep="")],
                e.F[q.plot,paste("F.",para.name[2],".0.bc",sep="")],
                l.bound.F.pwclus[q.plot,paste("F.",para.name[2],".0.bc",sep="")],
                u.bound.F.pwclus[q.plot,paste("F.",para.name[2],".1.bc",sep="")],
                e.F[q.plot,paste("F.",para.name[2],".1.bc",sep="")],
                l.bound.F.pwclus[q.plot,paste("F.",para.name[2],".1.bc",sep="")]),
          ys[q.plot],col=c("black","black","black","red","red","red"),lty=c(2,1,2,2,1,2),type="l",
          xlab="Quantile Index",ylab="Volume of Trade",main="Legal with pw Clustering");
  legend("topleft",legend=c("Legal=0","Legal=1","95% CB","95% CB"),
         col=c("black","red","black","red"),lty=c(1,1,2,2));

dev.off();

pdf(paste("Trade_qef",str.trim,".pdf",sep=""));
  para.name <- X.name[c(t.log2,t.d)];
  options(scipen=6);
  ylimlist <- rbind(c(-100000,0),c(0,180000));
  
  # Without pairwise clustering #
  
  curve(QF[[paste("F.",para.name[1],".1.bc",sep="")]](x)-QF[[paste("F.",para.name[1],".0.bc",sep="")]](x),0.5,0.95,
        ylim=ylimlist[1,],ylab="Diff in Volume of Trade", xlab="Quantile Index",main="Log Distance"); 
  # ylim=
  # range(c(u.qef[[paste("F.",para.name[1],".1.bc",sep="")]](c(0.5,0.95))-l.qef[[paste("F.",para.name[1],".0.bc",sep="")]](c(0.5,0.95)),
  # l.qef[[paste("F.",para.name[1],".1.bc",sep="")]](c(0.5,0.95))-u.qef[[paste("F.",para.name[1],".0.bc",sep="")]](c(0.5,0.95)))),
  curve(u.qef[[paste("F.",para.name[1],".1.bc",sep="")]](x)-l.qef[[paste("F.",para.name[1],".0.bc",sep="")]](x),0.5,0.95,add=T,lty=2);
  curve(l.qef[[paste("F.",para.name[1],".1.bc",sep="")]](x)-u.qef[[paste("F.",para.name[1],".0.bc",sep="")]](x),0.5,0.95,add=T,lty=2);
  curve(QF.pois[[paste("F.",para.name[1],".1",sep="")]](x)-QF.pois[[paste("F.",para.name[1],".0",sep="")]](x),0.5,0.95,add=T,col=4);
  abline(h=0, col = "grey");
#   abline(h=avg.eff.bc[para.name[1]]);
#   abline(h=u.bound.ae[para.name[1]], lty=2);
#   abline(h=l.bound.ae[para.name[1]], lty=2);
#   abline(h=avg.eff.pois.df[1],col=4);
  legend("bottomleft", c('DR-BC','Poisson','95% CB'), col = c(1,4,1), lty = c(1,1,2));
  
  curve(QF[[paste("F.",para.name[2],".1.bc",sep="")]](x)-QF[[paste("F.",para.name[2],".0.bc",sep="")]](x),0.5,0.95,
        ylim=ylimlist[2,],ylab="Diff in Volume of Trade", xlab="Quantile Index",main="Legal");
  curve(u.qef[[paste("F.",para.name[2],".1.bc",sep="")]](x)-l.qef[[paste("F.",para.name[2],".0.bc",sep="")]](x),0.5,0.95,add=T,lty=2);
  curve(l.qef[[paste("F.",para.name[2],".1.bc",sep="")]](x)-u.qef[[paste("F.",para.name[2],".0.bc",sep="")]](x),0.5,0.95,add=T,lty=2);
  curve(QF.pois[[paste("F.",para.name[2],".1",sep="")]](x)-QF.pois[[paste("F.",para.name[2],".0",sep="")]](x),0.5,0.95,add=T,col=4);
  abline(h=0, col = "grey");
#   abline(h=avg.eff.bc[para.name[2]]);
#   abline(h=u.bound.ae[para.name[2]], lty=2);
#   abline(h=l.bound.ae[para.name[2]], lty=2);
#   abline(h=avg.eff.pois.df[2],col=4);
  legend("topleft", c('DR-BC','Poisson','95% CB'), col = c(1,4,1), lty = c(1,1,2));
  
  
  # With pairwise clustering #
  
  curve(QF[[paste("F.",para.name[1],".1.bc",sep="")]](x)-QF[[paste("F.",para.name[1],".0.bc",sep="")]](x),0.5,0.95,
        ylim=ylimlist[1,],ylab="Diff in Volume of Trade", xlab="Quantile Index",main="Log Distance with pw Clustering");
  curve(u.qef.pwclus[[paste("F.",para.name[1],".1.bc",sep="")]](x)-l.qef.pwclus[[paste("F.",para.name[1],".0.bc",sep="")]](x),0.5,0.95,add=T,lty=2);
  curve(l.qef.pwclus[[paste("F.",para.name[1],".1.bc",sep="")]](x)-u.qef.pwclus[[paste("F.",para.name[1],".0.bc",sep="")]](x),0.5,0.95,add=T,lty=2);
  curve(QF.pois[[paste("F.",para.name[1],".1",sep="")]](x)-QF.pois[[paste("F.",para.name[1],".0",sep="")]](x),0.5,0.95,add=T,col=4);
  abline(h=0, col = "grey");
#   abline(h=avg.eff.bc[para.name[1]]);
#   abline(h=u.bound.ae.pwclus[para.name[1]], lty=2);
#   abline(h=l.bound.ae.pwclus[para.name[1]], lty=2);
#   abline(h=avg.eff.pois.df[1],col=4);
  legend("bottomleft", c('DR-BC','Poisson','95% CB'), col = c(1,4,1), lty = c(1,1,2));
  
  curve(QF[[paste("F.",para.name[2],".1.bc",sep="")]](x)-QF[[paste("F.",para.name[2],".0.bc",sep="")]](x),0.5,0.95,
        ylim=ylimlist[2,],ylab="Diff in Volume of Trade", xlab="Quantile Index",main="Legal with pw Clustering");
  curve(u.qef.pwclus[[paste("F.",para.name[2],".1.bc",sep="")]](x)-l.qef.pwclus[[paste("F.",para.name[2],".0.bc",sep="")]](x),0.5,0.95,add=T,lty=2);
  curve(l.qef.pwclus[[paste("F.",para.name[2],".1.bc",sep="")]](x)-u.qef.pwclus[[paste("F.",para.name[2],".0.bc",sep="")]](x),0.5,0.95,add=T,lty=2);
  curve(QF.pois[[paste("F.",para.name[2],".1",sep="")]](x)-QF.pois[[paste("F.",para.name[2],".0",sep="")]](x),0.5,0.95,add=T,col=4);
  abline(h=0, col = "grey");
#   abline(h=avg.eff.bc[para.name[2]]);
#   abline(h=u.bound.ae.pwclus[para.name[2]], lty=2);
#   abline(h=l.bound.ae.pwclus[para.name[2]], lty=2);
#   abline(h=avg.eff.pois.df[2],col=4);
  legend("topleft", c('DR-BC','Poisson','95% CB'), col = c(1,4,1), lty = c(1,1,2));

dev.off();